# Python
from __future__ import unicode_literals

# Django
from django.conf import settings

# Django-Site-Utils
from . import defaults


def get_site_utils_setting(name, merge_dicts=False):
    name = name.upper()
    try:
        default_setting = getattr(defaults, name)
        setting = getattr(settings, name, None)
        if merge_dicts:
            setting_dict = dict(default_setting.items())
            if setting:
                setting_dict.update(setting.items())
            return setting_dict
        else:
            return setting or default_setting
    except AttributeError:
        raise RuntimeError('invalid setting name: {}'.format(name))



# Python
import threading

_runserver_addrport_patched = threading.Event()
_wsgi_handler_keep_alive_patched = threading.Event()


def patch_runserver_addrport():
    if _runserver_addrport_patched.is_set():
        return
    _runserver_addrport_patched.set()

    # Use the addr/port defined in settings for runserver if not specified on the command line.
    from django.conf import settings
    default_addr = getattr(settings, 'RUNSERVER_DEFAULT_ADDR', '127.0.0.1')
    default_port = getattr(settings, 'RUNSERVER_DEFAULT_PORT', 8000)

    from django.core.management.commands import runserver as core_runserver
    original_handle = core_runserver.Command.handle

    def handle(self, *args, **options):
        if not options.get('addrport'):
            options['addrport'] = '%s:%d' % (default_addr, int(default_port))
        elif options.get('addrport').isdigit():
            options['addrport'] = '%s:%d' % (default_addr, int(options['addrport']))
        return original_handle(self, *args, **options)

    core_runserver.Command.handle = handle


def patch_wsgi_handler_keep_alive():
    if _wsgi_handler_keep_alive_patched.is_set():
        return
    _wsgi_handler_keep_alive_patched.set()

    # Patch Django's HTTP server to support keep-alive connections.
    from django.core.servers.basehttp import ServerHandler, WSGIRequestHandler
    original_handle_error = ServerHandler.handle_error
    original_close = ServerHandler.close

    def handle_error(self):
        if not self.request_handler.close_connection:
            self.request_handler.close_connection = True
        original_handle_error(self)

    def close(self):
        try:
            if not self.request_handler.close_connection:
                if self.headers and self.headers.get('connection') == 'close':
                    self.request_handler.close_connection = True
        finally:
            original_close(self)

    ServerHandler.handle_error = handle_error
    ServerHandler.close = close

    original_wsgi_handle = WSGIRequestHandler.handle

    def wsgi_handle(self):
        self.close_connection = True
        original_wsgi_handle(self)
        while not self.close_connection:
            original_wsgi_handle(self)

    WSGIRequestHandler.handle = wsgi_handle



# Django
from django.contrib import admin


class InlineSelectRelatedMixin(object):

    inline_select_related = ()

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if self.inline_select_related:
            qs = qs.select_related(*self.inline_select_related)
        return qs


class InlinePrefetchRelatedMixin(object):

    inline_prefetch_related = ()

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if self.inline_prefetch_related:
            qs = qs.prefetch_related(*self.inline_prefetch_related)
        return qs


class InlineMixins(InlineSelectRelatedMixin, InlinePrefetchRelatedMixin):
    pass


class StackedInline(InlineMixins, admin.StackedInline):
    pass


class TabularInline(InlineMixins, admin.TabularInline):
    pass


class AdminAddChangeDeleteMixin(object):

    can_add = True
    can_change = True
    can_delete = True

    def has_add_permission(self, request):
        return super().has_add_permission(request) if self.can_add else False

    def has_change_permission(self, request, obj=None):
        return super().has_change_permission(request, obj) if self.can_change else False

    def has_delete_permission(self, request, obj=None):
        return super().has_delete_permission(request, obj) if self.can_delete else False


class AdminListPrefetchRelatedMixin(object):

    list_prefetch_related = ()

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if self.list_prefetch_related:
            qs = qs.prefetch_related(*self.list_prefetch_related)
        return qs


class AdminAddFormMixin(object):

    add_form = None

    def get_form(self, request, obj=None, **kwargs):
        if obj is None and self.add_form:
            kwargs.setdefault('form', self.add_form)
        return super().get_form(request, obj, **kwargs)


class AdminAddViewFieldsMixin(object):

    add_fields = None
    add_fieldsets = None
    view_fields = None
    view_fieldsets = None

    def get_fields(self, request, obj=None):
        if obj is None and self.add_fields is not None:
            return self.add_fields
        elif obj is not None and self.view_fields is not None and not self.has_change_permission(request, obj):
            return self.view_fields
        else:
            return super().get_fields(request, obj)

    def get_fieldsets(self, request, obj=None):
        if obj is None and self.add_fieldsets is not None:
            return self.add_fieldsets
        elif obj is not None and self.view_fieldsets is not None and not self.has_change_permission(request, obj):
            return self.view_fieldsets
        else:
            return super().get_fieldsets(request, obj)


class AdminAddViewInlinesMixin(object):

    add_inlines = None
    view_inlines = None

    def get_inlines(self, request, obj=None):
        if obj is None and self.add_inlines is not None:
            return self.add_inlines
        elif obj is not None and self.view_inlines is not None and not self.has_change_permission(request, obj):
            return self.view_inlines
        else:
            return self.inlines

    def get_inline_instances(self, request, obj=None):
        inline_instances = []
        for inline_class in self.get_inlines(request, obj):
            inline = inline_class(self.model, self.admin_site)
            if request:
                inline_has_add_permission = inline.has_add_permission(request, obj)
                if not inline.has_view_or_change_permission(request, obj):
                    continue
                if not inline_has_add_permission:
                    inline.max_num = 0
            inline_instances.append(inline)
        return inline_instances


class AdminMixins(AdminAddChangeDeleteMixin, AdminListPrefetchRelatedMixin, AdminAddFormMixin, AdminAddViewFieldsMixin, AdminAddViewInlinesMixin):
    pass


class ModelAdmin(AdminMixins, admin.ModelAdmin):
    pass



# Python
from __future__ import unicode_literals

# Django
from django.apps import AppConfig
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _


class SiteUtilsConfig(AppConfig):

    name = 'site_utils'
    verbose_name = _('Site Utilities')

    def ready(self):
        from .settings import get_site_utils_setting
        site_patch_functions = []
        for func_import in get_site_utils_setting('SITE_PATCHES'):
            site_patch_functions.append(import_string(func_import))
        for func in site_patch_functions:
            func()



# Python
from __future__ import unicode_literals

# Django
from django.utils.translation import gettext_lazy as _


SITE_CLEANUP_FUNCTIONS = (
    'site_utils.cleanup.cleanup_stale_content_types',
    'site_utils.cleanup.cleanup_unused_database_tables',
)

SITE_NOTIFY_DEFAULT_RECIPIENTS = ('admins',)

SITE_NOTIFY_SUBJECT_TEMPLATE = 'site_utils/notify_subject.txt'

SITE_NOTIFY_BODY_TEMPLATE = 'site_utils/notify_body.txt'

SITE_UPDATE_COMMANDS = {
    'default': [
        'check',
        ('migrate', [], {'fake_initial': True}),
        ('collectstatic', [], {}, 'staticfiles'),
    ],
}

SITE_ERROR_MESSAGES = {
    400: (
        _('Bad Request'),
        _('The request could not be understood by the server.'),
    ),
    403: (
        _('Forbidden'),
        _('You do not have permission to access the requested resource.'),
    ),
    404: (
        _('Not Found'),
        _('The requested page could not be found.'),
    ),
    500: (
        _('Server Error'),
        _('An internal server error has occurred.'),
    ),
    502: (
        _('Bad Gateway'),
        _('An invalid response was received from the upstream server.'),
    ),
    503: (
        _('Server Unavailable'),
        _('The server is currently unavailable.'),
    ),
    504: (
        _('Gateway Timeout'),
        _('Did not receive a timely response from the upstream server.'),
    ),
}

SITE_ERROR_TEMPLATES = [
    (r'', 'site_utils/error.html'),
]

SITE_PATCHES = (
    'site_utils.patches.patch_runserver_addrport',
    'site_utils.patches.patch_wsgi_handler_keep_alive',
)
